@@ -1,6 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989-1994, 2004-2011 Free Software Foundation, Inc.
+   Copyright (C) 1989-1994, 2004-2014, 2016-2017, 2020-2026 Free
+   Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -15,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "m4.h"
@@ -25,17 +26,21 @@
 #include <signal.h>
 
 #include "c-stack.h"
+#include "configmake.h"
 #include "ignore-value.h"
 #include "progname.h"
+#include "propername.h"
 #include "version-etc.h"
 
 #ifdef DEBUG_STKOVF
-# include "assert.h"
+# include <assert.h>
 #endif
 
-#define AUTHORS "Rene' Seindal"
+/* TRANSLATORS: This is a non-ASCII name: The first name is (with
+   Unicode escapes) "Ren\u00e9" or (with HTML entities) "Ren&eacute;".  */
+#define AUTHORS proper_name_utf8 ("Rene' Seindal", "Ren\xC3\xA9 Seindal")
 
-static void usage (int) M4_GNUC_NORETURN;
+static _Noreturn void usage (int);
 
 /* Enable sync output for /lib/cpp (-s).  */
 int sync_output = 0;
@@ -79,7 +84,7 @@ int retcode;
 struct macro_definition
 {
   struct macro_definition *next;
-  int code; /* D, U, s, t, '\1', or DEBUGFILE_OPTION.  */
+  int code;                     /* D, U, s, t, '\1', or DEBUGFILE_OPTION.  */
   const char *arg;
 };
 typedef struct macro_definition macro_definition;
@@ -97,8 +102,19 @@ m4_error (int status, int errnum, const char *format, ...)
   va_start (args, format);
   verror_at_line (status, errnum, current_line ? current_file : NULL,
                   current_line, format, args);
-  if (fatal_warnings && ! retcode)
+  if (fatal_warnings && !retcode)
     retcode = EXIT_FAILURE;
+  va_end (args);
+}
+
+void
+m4_failure (int errnum, const char *format, ...)
+{
+  va_list args;
+  va_start (args, format);
+  verror_at_line (EXIT_FAILURE, errnum, current_line ? current_file : NULL,
+                  current_line, format, args);
+  assume (false);
 }
 
 /*-------------------------------.
@@ -112,8 +128,20 @@ m4_error_at_line (int status, int errnum, const char *file, int line,
   va_list args;
   va_start (args, format);
   verror_at_line (status, errnum, line ? file : NULL, line, format, args);
-  if (fatal_warnings && ! retcode)
+  if (fatal_warnings && !retcode)
     retcode = EXIT_FAILURE;
+  va_end (args);
+}
+
+void
+m4_failure_at_line (int errnum, const char *file, int line,
+                    const char *format, ...)
+{
+  va_list args;
+  va_start (args, format);
+  verror_at_line (EXIT_FAILURE, errnum, line ? file : NULL,
+                  line, format, args);
+  assume (false);
 }
 
 #ifndef SIGBUS
@@ -131,8 +159,8 @@ m4_error_at_line (int status, int errnum, const char *file, int line,
 /* Pre-translated messages for program errors.  Do not translate in
    the signal handler, since gettext and strsignal are not
    async-signal-safe.  */
-static const char * volatile program_error_message;
-static const char * volatile signal_message[NSIG];
+static const char *volatile program_error_message;
+static const char *volatile signal_message[NSIG];
 
 /* Print a nicer message about any programmer errors, then exit.  This
    must be aysnc-signal safe, since it is executed as a signal
@@ -175,94 +203,104 @@ static void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
-    xfprintf (stderr, "Try `%s --help' for more information.\n", program_name);
+    {
+      xfprintf (stderr, _("Try `%s --help' for more information."),
+                program_name);
+      fputs ("\n", stderr);
+    }
   else
     {
-      xprintf ("Usage: %s [OPTION]... [FILE]...\n", program_name);
-      fputs ("\
+      xprintf (_("Usage: %s [OPTION]... [FILE]...\n"), program_name);
+      fputs (_("\
 Process macros in FILEs.  If no FILE or if FILE is `-', standard input\n\
 is read.\n\
-", stdout);
-      fputs ("\
-\n\
+"), stdout);
+      puts ("");
+      fputs (_("\
 Mandatory or optional arguments to long options are mandatory or optional\n\
 for short options too.\n\
-\n\
+"), stdout);
+      puts ("");
+      fputs (_("\
 Operation modes:\n\
       --help                   display this help and exit\n\
       --version                output version information and exit\n\
-", stdout);
-      xprintf ("\
+"), stdout);
+      fputs (_("\
   -E, --fatal-warnings         once: warnings become errors, twice: stop\n\
                                  execution at first error\n\
   -i, --interactive            unbuffer output, ignore interrupts\n\
   -P, --prefix-builtins        force a `m4_' prefix to all builtins\n\
   -Q, --quiet, --silent        suppress some warnings for builtins\n\
+"), stdout);
+      xprintf (_("\
       --warn-macro-sequence[=REGEXP]\n\
                                warn if macro definition matches REGEXP,\n\
                                  default %s\n\
-", DEFAULT_MACRO_SEQUENCE);
+"), DEFAULT_MACRO_SEQUENCE);
 #ifdef ENABLE_CHANGEWORD
-      fputs ("\
+      fputs (_("\
   -W, --word-regexp=REGEXP     use REGEXP for macro name syntax\n\
-", stdout);
+"), stdout);
 #endif
-      fputs ("\
-\n\
+      puts ("");
+      fputs (_("\
 Preprocessor features:\n\
   -D, --define=NAME[=VALUE]    define NAME as having VALUE, or empty\n\
   -I, --include=DIRECTORY      append DIRECTORY to include path\n\
   -s, --synclines              generate `#line NUM \"FILE\"' lines\n\
   -U, --undefine=NAME          undefine NAME\n\
-", stdout);
+"), stdout);
       puts ("");
       xprintf (_("\
 Limits control:\n\
   -g, --gnu                    override -G to re-enable GNU extensions\n\
   -G, --traditional            suppress all GNU extensions\n\
-  -H, --hashsize=PRIME         set symbol lookup hash table size [509]\n\
+  -H, --hashsize=PRIME         set symbol lookup hash table size [%d]\n\
   -L, --nesting-limit=NUMBER   change nesting limit, 0 for unlimited [%d]\n\
-"), nesting_limit);
+"), HASHMAX, nesting_limit);
       puts ("");
-      fputs ("\
+      fputs (_("\
 Frozen state files:\n\
   -F, --freeze-state=FILE      produce a frozen state on FILE at end\n\
   -R, --reload-state=FILE      reload a frozen state from FILE at start\n\
-", stdout);
-      fputs ("\
-\n\
+"), stdout);
+      puts ("");
+      fputs (_("\
 Debugging:\n\
   -d, --debug[=FLAGS]          set debug level (no FLAGS implies `aeq')\n\
       --debugfile[=FILE]       redirect debug and trace output to FILE\n\
                                  (default stderr, discard if empty string)\n\
   -l, --arglength=NUM          restrict macro tracing size\n\
   -t, --trace=NAME             trace NAME when it is defined\n\
-", stdout);
-      fputs ("\
-\n\
+"), stdout);
+      puts ("");
+      fputs (_("\
 FLAGS is any of:\n\
   a   show actual arguments\n\
   c   show before collect, after collect and after call\n\
   e   show expansion\n\
   f   say current input file name\n\
   i   show changes in input files\n\
+"), stdout);
+      fputs (_("\
   l   say current input line number\n\
   p   show results of path searches\n\
   q   quote values as necessary, with a or e flag\n\
   t   trace for all macro calls, not only traceon'ed\n\
   x   add a unique macro call id, useful with c flag\n\
   V   shorthand for all of the above flags\n\
-", stdout);
-      fputs ("\
-\n\
+"), stdout);
+      puts ("");
+      fputs (_("\
 If defined, the environment variable `M4PATH' is a colon-separated list\n\
 of directories included after any specified by `-I'.\n\
-", stdout);
-      fputs ("\
-\n\
+"), stdout);
+      puts ("");
+      fputs (_("\
 Exit status is 0 for success, 1 for failure, 63 for frozen file version\n\
 mismatch, or whatever value was passed to the m4exit macro.\n\
-", stdout);
+"), stdout);
       emit_bug_reporting_address ();
     }
   exit (status);
@@ -277,19 +315,21 @@ mismatch, or whatever value was passed to the m4exit macro.\n\
 enum
 {
   DEBUGFILE_OPTION = CHAR_MAX + 1,      /* no short opt */
-  DIVERSIONS_OPTION,                    /* not quite -N, because of message */
-  WARN_MACRO_SEQUENCE_OPTION,           /* no short opt */
+  DIVERSIONS_OPTION,            /* not quite -N, because of message */
+  WARN_MACRO_SEQUENCE_OPTION,   /* no short opt */
 
-  HELP_OPTION,                          /* no short opt */
-  VERSION_OPTION                        /* no short opt */
+  HELP_OPTION,                  /* no short opt */
+  VERSION_OPTION                /* no short opt */
 };
 
-static const struct option long_options[] =
-{
+static const struct option long_options[] = {
   {"arglength", required_argument, NULL, 'l'},
   {"debug", optional_argument, NULL, 'd'},
   {"define", required_argument, NULL, 'D'},
-  {"error-output", required_argument, NULL, 'o'}, /* FIXME: deprecate in 2.0 */
+
+  /* FIXME: deprecate in 2.0 */
+  {"error-output", required_argument, NULL, 'o'},
+
   {"fatal-warnings", no_argument, NULL, 'E'},
   {"freeze-state", required_argument, NULL, 'F'},
   {"gnu", no_argument, NULL, 'g'},
@@ -305,16 +345,19 @@ static const struct option long_options[] =
   {"trace", required_argument, NULL, 't'},
   {"traditional", no_argument, NULL, 'G'},
   {"undefine", required_argument, NULL, 'U'},
+#ifdef ENABLE_CHANGEWORD
   {"word-regexp", required_argument, NULL, 'W'},
+#endif
 
   {"debugfile", optional_argument, NULL, DEBUGFILE_OPTION},
   {"diversions", required_argument, NULL, DIVERSIONS_OPTION},
-  {"warn-macro-sequence", optional_argument, NULL, WARN_MACRO_SEQUENCE_OPTION},
+  {"warn-macro-sequence", optional_argument, NULL,
+   WARN_MACRO_SEQUENCE_OPTION},
 
   {"help", no_argument, NULL, HELP_OPTION},
   {"version", no_argument, NULL, VERSION_OPTION},
 
-  { NULL, 0, NULL, 0 },
+  {NULL, 0, NULL, 0},
 };
 
 /* Process a command line file NAME, and return true only if it was
@@ -333,7 +376,7 @@ process_file (const char *name)
   else
     {
       char *full_name;
-      FILE *fp = m4_path_search (name, &full_name);
+      FILE *fp = m4_path_search (name, false, &full_name);
       if (fp == NULL)
         {
           error (0, errno, _("cannot open `%s'"), name);
@@ -354,9 +397,9 @@ process_file (const char *name)
    '-' forces getopt_long to hand back file names as arguments to opt
    '\1', rather than reordering the command line.  */
 #ifdef ENABLE_CHANGEWORD
-#define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:W:d::egil:o:st:"
+# define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:W:d::egil:o:st:"
 #else
-#define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:d::egil:o:st:"
+# define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:d::egil:o:st:"
 #endif
 
 int
@@ -378,6 +421,13 @@ main (int argc, char *const *argv)
 
   set_program_name (argv[0]);
   retcode = EXIT_SUCCESS;
+  setlocale (LC_ALL, "");
+  /* m4 1.4.x does not want locale-aware decimal separators in the
+     format builtin; easiest is to override the user's choice of
+     LC_NUMERIC. */
+  setlocale (LC_NUMERIC, "C");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
   atexit (close_stdin);
 
   include_init ();
@@ -420,7 +470,7 @@ main (int argc, char *const *argv)
     if (crash)
       {
         if (!strtol (crash, NULL, 10))
-          ++*(int *) 8;
+          ++ * (int *) 8;
         assert (false);
         abort ();
       }
@@ -443,7 +493,8 @@ main (int argc, char *const *argv)
         /* Compatibility junk: options that other implementations
            support, but which we ignore as no-ops and don't list in
            --help.  */
-        error (0, 0, _("warning: `m4 -%c' may be removed in a future release"),
+        error (0, 0,
+               _("warning: `m4 -%c' may be removed in a future release"),
                optchar);
         break;
 
@@ -476,7 +527,7 @@ main (int argc, char *const *argv)
         break;
 
       case 'E':
-        if (! fatal_warnings)
+        if (!fatal_warnings)
           fatal_warnings = true;
         else
           warning_status = EXIT_FAILURE;
@@ -533,7 +584,7 @@ main (int argc, char *const *argv)
 
       case 'e':
         error (0, 0, _("warning: `m4 -e' is deprecated, use `-i' instead"));
-        /* fall through */
+        FALLTHROUGH;
       case 'i':
         interactive = true;
         break;
@@ -558,10 +609,10 @@ main (int argc, char *const *argv)
         break;
 
       case WARN_MACRO_SEQUENCE_OPTION:
-         /* Don't call set_macro_sequence here, as it can exit.
-            --warn-macro-sequence sets optarg to NULL (which uses the
-            default regexp); --warn-macro-sequence= sets optarg to ""
-            (which disables these warnings).  */
+        /* Don't call set_macro_sequence here, as it can exit.
+           --warn-macro-sequence sets optarg to NULL (which uses the
+           default regexp); --warn-macro-sequence= sets optarg to ""
+           (which disables these warnings).  */
         macro_sequence = optarg;
         break;
 
@@ -579,7 +630,8 @@ main (int argc, char *const *argv)
 
   /* Do the basic initializations.  */
   if (debugfile && !debug_set_output (debugfile))
-    M4ERROR ((warning_status, errno, "cannot set debug file `%s'", debugfile));
+    M4ERROR ((warning_status, errno, _("cannot set debug file `%s'"),
+              debugfile));
 
   input_init ();
   output_init ();
@@ -617,17 +669,20 @@ main (int argc, char *const *argv)
             char *macro_value = strchr (macro_name, '=');
             if (macro_value)
               *macro_value++ = '\0';
-            define_user_macro (macro_name, macro_value, SYMBOL_INSERT);
+            define_user_macro (macro_name, strlen (macro_name), macro_value,
+                               macro_value ? strlen (macro_value) : 0,
+                               SYMBOL_INSERT);
             free (macro_name);
           }
           break;
 
         case 'U':
-          lookup_symbol (defines->arg, SYMBOL_DELETE);
+          lookup_symbol (defines->arg, strlen (defines->arg), SYMBOL_DELETE);
           break;
 
         case 't':
-          sym = lookup_symbol (defines->arg, SYMBOL_INSERT);
+          sym = lookup_symbol (defines->arg, strlen (defines->arg),
+                               SYMBOL_INSERT);
           SYMBOL_TRACED (sym) = true;
           break;
 
@@ -642,7 +697,7 @@ main (int argc, char *const *argv)
 
         case DEBUGFILE_OPTION:
           if (!debug_set_output (defines->arg))
-            M4ERROR ((warning_status, errno, "cannot set debug file `%s'",
+            M4ERROR ((warning_status, errno, _("cannot set debug file `%s'"),
                       debugfile ? debugfile : _("stderr")));
           break;
 
